﻿local addon = LibStub("AceAddon-3.0"):NewAddon("GCD", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GCD")
local ACD = LibStub("AceConfigDialog-3.0")
local dbVersion = 50

addon:SetDefaultModuleState(false)
addon:SetDefaultModuleLibraries("AceEvent-3.0")

local defaults, options
local BOOKTYPE_SPELL = BOOKTYPE_SPELL
local showRequests = {}

local cursorOffset = {
	x = 10,
	y = -10
}

function addon:Show(name)
	showRequests[name] = true
	self.anchor:Show()
end

function addon:Hide(name)
	showRequests[name] = false
	
	local hide = true
	for _,v in pairs(showRequests) do
		if v then
			hide = false
		end
	end
	if hide then
		self.anchor:Hide()
	end
end

function addon:GetSpellPosInSpellbook(spellName)
	local spellNum = nil
	for tab = 1, 4 do
		local _, _, offset, numSpells = GetSpellTabInfo(tab)
		for i = (1+offset), (offset+numSpells) do
			local spell = GetSpellName(i, BOOKTYPE_SPELL)
			if strlower(spell) == strlower(spellName) then
				spellNum = i
				break
			end
		end
	end
	return spellNum;
end

function addon:InitializeDefaults()
	defaults = {
		char = {
			attachToMouse = true,
			talentSpecs = {
				primary = true,
				secondary = true
			},
			modules = {}
		},
		profile = {
			position = {
				x = 400,
				y = 400
			},
			modules = {}
		},
		global = {
			version = dbVersion
		}
	}

	for name, module in self:IterateModules() do
		defaults.char.modules[name] = true
	end
end

function addon:InitializeOptions()
	options = {
		name = "GCD",
		type = "group",
		args = {
			unlock = {
				name = L["Unlock"],
				type = "execute",
				func = function() addon:Unlock() end,
				order = 1
			},
			info = {
				name = L["All specified colors are vertex colors! (see wowwiki.com)"],
				type = "description",
				order = 2
			},
			general = {
				name = L["General"],
				type = "group",
				args = {
					position = {
						name = L["Position"],
						type = "header",
						order = 0
					},
					mouse = {
						name = L["Attach to cursor"],
						type = "toggle",
						get = function(info) return self.db.char.attachToMouse end,
						set = function(info,val)
								self.db.char.attachToMouse = val
							end,
						order = 1
					},
					x = {
						name = L["Left"],
						type = "input",
						get = function(info) return tostring(self.db.profile.position.x) end,
						set = function(info, val)
									self.db.profile.position.x = tonumber(val)
									self:ApplyOptions()
								end,
						validate = function(_, value)
											if tonumber(value) then
												return true
											else
												return L["Left has to be a number!"]
											end
										end,
						order = 2
					},
					y = {
						name = L["Bottom"],
						type = "input",
						get = function(info) return tostring(self.db.profile.position.y) end,
						set = function(info, val)
									self.db.profile.position.y = tonumber(val)
									self:ApplyOptions()
								end,
						validate = function(_, value)
											if tonumber(value) then
												return true
											else
												return L["Bottom has to be a number!"]
											end
										end,
						order = 3
					},
					specs = {
						name = L["Talent Specs"],
						type = "header",
						order = 10
					},
					primary = {
						name = L["Primary"],
						type = "toggle",
						get = function(info) return self.db.char.talentSpecs.primary end,
						set = function(info,val)
								self.db.char.talentSpecs.primary = val
								self:ACTIVE_TALENT_GROUP_CHANGED()
							end,
						order = 11
					},
					secondary = {
						name = L["Secondary"],
						type = "toggle",
						get = function(info) return self.db.char.talentSpecs.secondary end,
						set = function(info,val)
								self.db.char.talentSpecs.secondary = val
								self:ACTIVE_TALENT_GROUP_CHANGED()
							end,
						order = 12
					},
				},
				order = 10
			}
		}
	}
	local i = 2
	for name, module in self:IterateModules() do
		local modOptions = module:GetOptions()
		modOptions.order = i*10
		modOptions.args.enabled = {
			name = L["Enabled"],
			type = "toggle",
			get = function(info) return self.db.char.modules[name] end,
			set = function(info,val)
					self.db.char.modules[name] = val
					if val then
						self:GetModule(name):Enable()
					else
						self:GetModule(name):Disable()
					end
				end,
			order = 0
		}
		options.args[name] = modOptions
		i = i + 1
	end
end

local function OnUpdate(self, elapsed)
	local x, y
	if addon.db.char.attachToMouse then
		x, y = GetCursorPosition(UIParent);
	else
		x, y = addon.db.profile.position.x, addon.db.profile.position.y
	end
	self:ClearAllPoints()
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', (x + cursorOffset.x) / self:GetEffectiveScale(), (y + cursorOffset.y) / self:GetEffectiveScale())
end

function addon:FixDatabase()
	-- Does nothing yet, just some prep for the future
	self.db.global.version = dbVersion
end

function addon:OnInitialize()
	self:InitializeDefaults()
	self.db = LibStub("AceDB-3.0"):New("GCDDB", defaults)
	self:FixDatabase()
	
	self:InitializeOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GCD", options)
	ACD:SetDefaultSize("GCD", 640, 480)
	self:RegisterChatCommand("gcd", self.OpenConfig)
	self:ApplyOptions()
end

function addon:OnEnable()
	local activeSpec = GetActiveTalentGroup(false, false)
	if (activeSpec == 1 and self.db.char.talentSpecs.primary) or (activeSpec == 2 and self.db.char.talentSpecs.secondary) then
		for name, _ in self:IterateModules() do
			if self.db.char.modules[name] then
				self:EnableModule(name)
			end
		end
	end
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	return true
end

function addon:OnDisable()
	if self.anchor then self.anchor:Hide() end
	for _, module in self:IterateModules() do
		if module:IsEnabled() then
			module:Disable()
		end
	end
	self:UnregisterAllEvents()
	return true
end

function addon:ACTIVE_TALENT_GROUP_CHANGED()
	local activeSpec = GetActiveTalentGroup(false, false)
	if (activeSpec == 1 and self.db.char.talentSpecs.primary) or (activeSpec == 2 and self.db.char.talentSpecs.secondary) then
		self:OnEnable()
	else
		self:OnDisable()
		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	end
end

function addon:ApplyOptions()
	local anchor = self.anchor or CreateFrame("Frame")
	anchor:Hide()
	anchor:ClearAllPoints()
	anchor:SetFrameStrata("HIGH")
	anchor:SetScript('OnUpdate', OnUpdate)
	anchor:SetWidth(64)
	anchor:SetHeight(64)
	
	self.anchor = anchor
		
	for name, module in self:IterateModules() do
	   if self.db.char[name] and module:IsDisabled() then
			module:Enable()
	   elseif not self.db.char[name] and module:IsEnabled() then
			module:Disable()
	   end
	   module:ApplyOptions()
	end
end

function addon:OpenConfig()
	ACD:Open("GCD")
end

--[[
		Unlock
]]

local cursorBackdrop = {bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
							edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
							tile = true, tileSize = 16, edgeSize = 16, 
							insets = { left = 4, right = 4, top = 4, bottom = 4 }}
local cursorFrame

function addon:Unlock()
	local unlocked = false
	for name, module in self:IterateModules() do
		if self.db.char.modules[name] and module.Unlock then
			unlocked = true
		end
	end
	if unlocked then
		local cursor = cursorFrame or CreateFrame("Frame")
		cursor:SetWidth(20)
		cursor:SetHeight(20)
		cursor:SetBackdrop(cursorBackdrop)
		cursor:SetBackdropColor(255,0,0,1)
		cursor:SetFrameStrata('HIGH')
		cursor:SetPoint("CENTER", UIParent, "BOTTOMLEFT", (self.db.profile.position.x + cursorOffset.x) / cursor:GetEffectiveScale(), (self.db.profile.position.y + cursorOffset.y) / cursor:GetEffectiveScale())
		cursor:SetScript("OnMouseDown", cursor.StartMoving)
		cursor:SetScript("OnMouseUp", function(self, button)
											self:StopMovingOrSizing()
											if button == "RightButton" then
												local x, y = self:GetCenter()
												addon.db.profile.position.x, addon.db.profile.position.y = x - cursorOffset.x, y - cursorOffset.y
												self:Hide()
												for name, module in addon:IterateModules() do
													if addon.db.char.modules[name] and module.Lock then
														module:Lock()
													end
												end
											end
										end)
		cursor:SetScript("OnEnter", function(self)
										GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
										GameTooltip:SetText("Right Click")
										GameTooltip:AddLine("to lock the icons and save their positions", 1, 1, 1)
										GameTooltip:Show()
									end)
		cursor:SetScript("OnLeave", function(self)
										GameTooltip:Hide()
									end)
		cursor:SetMovable(true)
		cursor:EnableMouse(true)
		cursor:Show()
		
		for name, module in self:IterateModules() do
			if self.db.char.modules[name] and module.Unlock then
				module:Unlock(cursor)
			end
		end
		
		ACD:Close("GCD")
		return true
	else
		return L["All unlockable modules are disabled!"]
	end
end
